/*
 * Copyright (c) 2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Definitions.java 2990 2006-08-06 20:23:49Z jmettraux $
 */

//
// Feed.java
//
// john.mettraux@openwfe.org
//
// generated with 
// jtmpl 1.1.01 2004/05/19 (john.mettraux@openwfe.org)
//

package openwfe.org.syndication;

import openwfe.org.Utils;


/**
 * A generic syndication feed (more Atom than RSS inspired though).
 *
 * <p><font size=2>CVS Info :
 * <br>$Author$
 * <br>$Id$ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public class Feed

    extends SyndObject

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(Feed.class.getName());

    //
    // CONSTANTS & co

    //
    // FIELDS

    private int maxSize = -1;
    private java.util.List entries = null;

    //
    // CONSTRUCTORS

    public Feed ()
    {
        super();

        this.entries = new java.util.LinkedList();

        this.setUpdated(new java.util.Date());
            //
            // could get overriden while decoding a stored feed
    }

    /**
     * Creates an empty feed with a given max size. If
     * the max size is negative, there'll be no limit
     * to the number of entry this feed will contain.
     */
    public Feed (final int maxSize)
    {
        this();

        this.maxSize = maxSize;
        if (this.maxSize < 0) this.maxSize = -1;
    }

    //
    // METHODS

    public java.util.Iterator iterator ()
    {
        return this.entries.iterator();
    }

    /**
     * Adds an entry, makes sure to override any previous one with the 
     * same id.
     */
    public synchronized void addEntry (final Entry e)
    {
        final Entry eToRemove = fetch(e.getId());

        if (eToRemove != null)
            this.entries.remove(eToRemove);

        this.entries.add(0, e);

        if (maxSize < 1) return;

        while (this.entries.size() > maxSize)
            this.entries.remove(this.entries.size()-1);
    }

    /**
     * Removes the given entry from the feed.
     */
    public synchronized void removeEntry (final String entryId)
    {
        final Entry eToRemove = fetch(entryId);

        if (eToRemove != null)
            this.entries.remove(eToRemove);

        /*
        if (log.isDebugEnabled())
        {
            if (eToRemove != null)
                log.debug("removeEntry() removed entry '"+entryId+"'");
            else
                log.debug("removeEntry() didn't remove entry '"+entryId+"'");
        }
        */
    }

    private Entry fetch (final String entryId)
    {
        final java.util.Iterator it = this.entries.iterator();
        while (it.hasNext())
        {
            final Entry entry = (Entry)it.next();

            if (entry.getId().equals(entryId))
                return entry;
        }

        return null;
    }

    public boolean equals (final Object o)
    {
        //Utils.logStackTrace(log, "equals()");

        if (o == null) return false;

        if ( ! (o instanceof Feed)) return false;

        final Feed other = (Feed)o;

        // (do not check maxsize)

        //log.debug("equals() soEquals ? "+this.soEquals(other));

        if ( ! this.soEquals(other)) return false;

        //log.debug("equals() this.entries.size() : "+this.entries.size());

        return Utils.listEquals(this.entries, other.entries);
    }

    public int hashCode ()
    {
        int i = this.getId().hashCode() * 100;

        if (this.getUpdated() == null)
            i = i * (-1);
        else
            i += this.getUpdated().hashCode();

        if (this.entries != null)
        {
            final java.util.Iterator it = this.entries.iterator();

            while (it.hasNext())
                i += ((Entry)it.next()).hashCode();
        }

        return i;
    }

    //
    // STATIC METHODS

}
