/*
 * Copyright (c) 2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Definitions.java 2990 2006-08-06 20:23:49Z jmettraux $
 */

//
// SyndObject.java
//
// john.mettraux@openwfe.org
//
// generated with 
// jtmpl 1.1.01 2004/05/19 (john.mettraux@openwfe.org)
//

package openwfe.org.syndication;

import openwfe.org.Utils;
import openwfe.org.time.Time;


/**
 * A base class for Feed and Entry.
 *
 * <p><font size=2>CVS Info :
 * <br>$Author$
 * <br>$Id$ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public abstract class SyndObject
{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(SyndObject.class.getName());

    //
    // CONSTANTS & co

    //
    // FIELDS

    // required

    private String id = null;
    private Content title = null;
    private java.util.Date updated = null;

    // recommended

    private Person author = null;
    private Link link = null;

    //
    // CONSTRUCTORS

    public SyndObject ()
    {
        super();
    }

    //
    // GETTERS and SETTERS

    public String getId () { return this.id; }
    public Content getTitle () { return this.title; }
    public java.util.Date getUpdated () { return this.updated; }
    public Person getAuthor () { return this.author; }
    public Link getLink () { return this.link; }

    public void setId (final String s) { this.id = s; }
    public void setTitle (final Content c) { this.title = c; }
    public void setUpdated (final java.util.Date d) { this.updated = d; }
    public void setAuthor (final Person p) { this.author = p; }
    public void setLink (final Link l) { this.link = l; }

    //
    // METHODS

    public void setTitle (final String s)
    {
        this.title = new Content(s);
    }

    /**
     * "Half an equals" : checks the equality just for the SyndObject fields;
     * the extending classes will use it but will also have to check for
     * equality of their own fields.
     */
    protected boolean soEquals (final SyndObject so)
    {
        //if (so == null) return false;
            //
            // not necessary : already checked in the extending classes

        /*
        log.debug("soEquals() id ? "+Utils.equals(this.id, so.id));
        log.debug("soEquals() title ? "+Utils.equals(this.title, so.title));
        log.debug("soEquals() author ? "+Utils.nequals(this.author, so.author));
        log.debug("soEquals() link ? "+Utils.nequals(this.link, so.link));
        */

        final String thisUpdated = Time.toRfcDate(this.updated);
        final String otherUpdated = Time.toRfcDate(so.updated);

        /*
        log.debug("soEquals() tClass  "+this.getClass().getName());
        log.debug("soEquals() oClass  "+so.getClass().getName());
        log.debug("soEquals() tId >"+this.id+"<");
        log.debug("soEquals() oId >"+so.id+"<");
        log.debug("soEquals() tUpdtd >"+thisUpdated+"<");
        log.debug("soEquals() oUpdtd >"+otherUpdated+"<");
        log.debug("soEquals() tUpdtd >"+this.updated+"<");
        log.debug("soEquals() oUpdtd >"+so.updated+"<");
        */

        if ( ! thisUpdated.equals(otherUpdated)) return false;

        return
            Utils.equals(this.id, so.id) &&
            //Utils.equals(this.title, so.title) &&
            Utils.nequals(this.author, so.author) &&
            Utils.nequals(this.link, so.link);
    }

    //
    // ABSTRACT METHODS

    /**
     * Forcing the extending classes to implement that method.
     */
    public abstract int hashCode ();

    /**
     * Forcing the extending classes to implement that method.
     */
    public abstract boolean equals (Object o);

    //
    // STATIC METHODS

}
