/*
 * Copyright (c) 2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Definitions.java 2990 2006-08-06 20:23:49Z jmettraux $
 */

//
// Atom10Coder.java
//
// john.mettraux@openwfe.org
//
// generated with 
// jtmpl 1.1.01 2004/05/19 (john.mettraux@openwfe.org)
//

package openwfe.org.syndication.impl;

import org.jdom.Text;
import org.jdom.Element;
import org.jdom.Document;
import org.jdom.Namespace;

import openwfe.org.xml.XmlUtils;
import openwfe.org.time.Time;
import openwfe.org.syndication.Feed;
import openwfe.org.syndication.Content;
import openwfe.org.syndication.Link;
import openwfe.org.syndication.Entry;
import openwfe.org.syndication.Person;
import openwfe.org.syndication.FeedCoder;
import openwfe.org.syndication.SyndObject;


/**
 * Turning the feed into Atom 1.0 XML.
 *
 * <p><font size=2>CVS Info :
 * <br>$Author$
 * <br>$Id$ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public class Atom10Coder

    implements FeedCoder

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(Atom10Coder.class.getName());

    //
    // CONSTANTS & co

    /**
     * The URI to the Atom 1.0 namespace.
     */
    protected final static String ATOM_NS 
        = "http://www.w3.org/2005/Atom";

    /**
     * An org.jdom.Namespace instance pointing
     * to the Atom 1.0 namespace.
     */
    protected final static Namespace ATOM_NSI
        = Namespace.getNamespace(ATOM_NS);

    protected final static String E_FEED = "feed";
    protected final static String E_ID = "id";
    protected final static String E_TITLE = "title";
    protected final static String E_UPDATED = "updated";
    protected final static String E_AUTHOR = "author";
    protected final static String E_NAME = "name";
    protected final static String E_URI = "uri";
    protected final static String E_LINK = "link";
    protected final static String E_ENTRY = "entry";
    protected final static String E_SUMMARY = "summary";
    protected final static String E_CONTENT = "content";

    protected final static String A_TYPE = "type";
    protected final static String A_HREF = "href";
    protected final static String A_REL = "rel";

    protected final static String TT_TEXT = "text";
        //
        // TT = Text Type

    protected final static java.util.Map TT_MAP = new java.util.HashMap();
    static
    {
        TT_MAP.put("application/xml", "text/xml");
        TT_MAP.put("text/plain", "text");
    }

    //
    // FIELDS

    //
    // CONSTRUCTORS

    public Atom10Coder ()
    {
        super();
    }

    //
    // METHODS from FeedCoder

    // ENCODING

    /**
     * Encodes a feed into an XML document.
     */
    public Document encode (final Feed f)
    {
        final Document d = new Document();

        final Element eFeed = encodeSyndObject(f, E_FEED);

        //
        // add entries

        final java.util.Iterator it = f.iterator();
        while (it.hasNext())
        {
            final Entry entry = (Entry)it.next();

            final Element eEntry = encodeEntry(entry);

            eFeed.addContent(eEntry);
        }

        d.setRootElement(eFeed);

        return d;
    }

    //
    // METHODS

    protected Element encodeSyndObject 
        (final SyndObject so, final String name)
    {
        final Element elt = new Element(name, ATOM_NS);

        final Element eId = new Element(E_ID, ATOM_NS);
        eId.setText(so.getId());
        elt.addContent(eId);

        elt.addContent(encodeContent(so.getTitle(), E_TITLE));

        // updated

        if (so.getUpdated() == null)
        {
            throw new IllegalArgumentException
                ("Missing 'updated' field for SyndObject of class "+
                  so.getClass().getName());
        }

        final Element eUpdated = new Element(E_UPDATED, ATOM_NS);
        eUpdated.setText(Time.toRfcDate(so.getUpdated()));
        elt.addContent(eUpdated);

        // author

        if (so.getAuthor() != null)
            elt.addContent(encodePerson(so.getAuthor(), E_AUTHOR));

        // link

        if (so.getLink() != null)
            elt.addContent(encodeLink(so.getLink(), E_LINK));

        return elt;
    }

    protected Element encodeEntry
        (final Entry e)
    {
        final Element elt = encodeSyndObject(e, E_ENTRY);

        if (e.getDescription() != null)
            elt.addContent(encodeSummary(e.getDescription()));

        if (e.getContents() != null && e.getContents().size() > 0)
        {
            // atom 1.0 allows 1 or 0 <content> element

            final Content con = (Content)e.getContents().get(0);

            elt.addContent(encodeContent(con, E_CONTENT));
        }

        return elt;
    }

    /**
     * A summary can only be of type "text", "html" or "xhtml", no
     * MIME types allowed.
     */
    protected Element encodeSummary
        (final Content c)
    {
        final Element elt = new Element(E_SUMMARY, ATOM_NS);

        elt.setAttribute(A_TYPE, translateSummaryType(c.getType()));

        elt.addContent(new Text(c.getValue().toString()));

        return elt;
    }

    protected Element encodeContent
        (final Content c, final String name)
    {
        final Element elt = new Element(name, ATOM_NS);

        String type = c.getType();
        if (type == null) type = TT_TEXT;

        elt.setAttribute(A_TYPE, type);

        if (type.endsWith("xml"))
        {
            try
            {
                org.jdom.Content con = null;

                if (c.getValue() instanceof org.jdom.Content)
                {
                    con = (org.jdom.Content)c.getValue();
                    if (con != null) con = (org.jdom.Content)con.clone();
                }
                else
                {
                    con = XmlUtils.extractXmlElement(c.getValue().toString());
                }

                con.detach();
                elt.addContent(con);
            }
            catch (final Throwable th)
            {
                log.warn("encodeText() failed to encode content", th);

                elt.setText("(failed to encode content : "+th+")");
            }
        }
        else
        {
            elt.setText(c.getValue().toString());
        }

        return elt;
    }

    protected String translateSummaryType (final String contentType)
    {
        if (contentType == null) return TT_TEXT;

        final String result = (String)TT_MAP.get(contentType);

        if (result != null) return result;

        return contentType;
    }

    protected Element encodePerson
        (final Person p, final String name)
    {
        final Element elt = new Element(name, ATOM_NS);

        final Element eName = new Element(E_NAME, ATOM_NS);
        eName.setText(p.getName());
        elt.addContent(eName);

        if (p.getUri() != null)
        {
            final Element eUri = new Element(E_URI, ATOM_NS);
            eUri.setText(p.getUri());
            elt.addContent(eUri);
        }

        return elt;
    }

    protected Element encodeLink
        (final Link l, final String name)
    {
        final Element elt = new Element(name, ATOM_NS);

        elt.setAttribute(A_HREF, l.getHref());

        if (l.getRel() != null)
            elt.setAttribute(A_REL, l.getRel());

        return elt;
    }

    // DECODING

    /**
     * Decodes a feed from an XML document.
     */
    public Feed decode (final Document d)
    {
        final Element eFeed = d.getRootElement();

        final Feed feed = new Feed();

        decodeSyndObject(eFeed, feed);

        //log.debug("decode() 0 feed.updated >"+feed.getUpdated()+"<");

        //
        // decode entries

        final java.util.Iterator it = eFeed
            .getChildren(E_ENTRY, ATOM_NSI)
            .iterator();

        while (it.hasNext())
        {
            final Entry entry = decodeEntry((Element)it.next());
            feed.addEntry(entry);
        }

        //log.debug("decode() 1 feed.updated >"+feed.getUpdated()+"<");

        return feed;
    }

    protected void decodeSyndObject (final Element elt, final SyndObject so)
    {
        final Element eId = elt.getChild(E_ID, ATOM_NSI);
        so.setId(XmlUtils.fetchTextContent(eId));

        final Element eTitle = elt.getChild(E_TITLE, ATOM_NSI);
        so.setTitle(decodeContent(eTitle));

        //log.debug("decodeSyndObject() title >"+so.getTitle()+"<");

        try
        {
            final Element eUpdated = elt.getChild(E_UPDATED, ATOM_NSI);

            so.setUpdated
                (new java.util.Date(Time.fromRfcDate(eUpdated.getTextTrim())));

            log.debug
                ("decodeSyndObject() so.class is "+so.getClass().getName());
            log.debug
                ("decodeSyndObject() updated >"+so.getUpdated()+"<");
        }
        catch (final java.text.ParseException e)
        {
            log.warn("decodeSyndObject() failed to parse 'updated' date", e);

            //so.setUpdated(new java.util.Date());
        }

        final Element eAuthor = elt.getChild(E_AUTHOR, ATOM_NSI);
        so.setAuthor(decodePerson(eAuthor));

        final Element eLink = elt.getChild(E_LINK, ATOM_NSI);
        so.setLink(decodeLink(eLink));
    }

    protected Content decodeContent (final Element elt)
    {
        if (elt == null) return null;

        final Content c = new Content();

        c.setType(elt.getAttributeValue(A_TYPE));

        log.debug
            ("decodeContent() elt.children.size = "+elt.getChildren().size());

        if (c.getType() != null && c.getType().endsWith("xml"))
            //
            // XML
        {
            c.setValue(elt.getChildren().get(0));
        }
        else
            //
            // not XML
        {
            if (elt.getChildren().size() > 0)
            {
                c.setValue
                    (XmlUtils.xmlToString((Element)elt.getChildren().get(0)));
            }
            else
            {
                c.setValue
                    (XmlUtils.fetchTextContent(elt));
            }
        }

        log.debug
            ("decodeContent() result >"+c.getValue()+"<");

        return c;
    }

    protected Entry decodeEntry (final Element elt)
    {
        final Entry entry = new Entry();

        decodeSyndObject(elt, entry);

        // summary

        entry.setDescription(decodeContent(elt.getChild(E_SUMMARY, ATOM_NSI)));

        // content

        final Content con = decodeContent(elt.getChild(E_CONTENT, ATOM_NSI));

        if (con != null)
            entry.getContents().add(con);

        return entry;
    }

    protected Person decodePerson (final Element elt)
    {
        if (elt == null) return null;

        final Person person = new Person();

        person.setName(elt.getChildTextTrim(E_NAME, ATOM_NSI));
        person.setUri(elt.getChildTextTrim(E_URI, ATOM_NSI));

        return person;
    }

    protected Link decodeLink (final Element elt)
    {
        final Link link = new Link();

        link.setHref(elt.getAttributeValue(A_HREF));
        link.setRel(elt.getAttributeValue(A_REL));

        return link;
    }

    //
    // STATIC METHODS

}
